package fer.hr.pipeline;

import fer.hr.preprocessing.binarisation.IBinarisation;
import fer.hr.preprocessing.greyscale.Greyscale;
import fer.hr.preprocessing.greyscale.IGreyscaleConvert;
import fer.hr.preprocessing.histogram.IHistogram;
import fer.hr.recognition.IRecognition;
import fer.hr.segmentation.ISegmentation;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * Created by jcular on 5/9/16.
 */
public class PipelineContext {
    private ISegmentation segmentation;
    private IBinarisation binarisation;
    private IRecognition recognition;
    private IHistogram histogram;
    private IGreyscaleConvert greyscaleConvert;

    public PipelineContext(ISegmentation segmentation, IBinarisation binarisation, IRecognition recognition, IHistogram histogram, IGreyscaleConvert greyscaleConvert) {
        this.segmentation = segmentation;
        this.recognition = recognition;
        this.histogram = histogram;
        this.binarisation = binarisation;
        this.greyscaleConvert = greyscaleConvert;
    }

    public String recognizeNumbers(String pathToPicture) throws IOException {
        BufferedImage bufferedImage = ImageIO.read(new File(String.valueOf(pathToPicture)));
        bufferedImage = greyscaleConvert.convertToGrey(bufferedImage);
        bufferedImage = binarisation.convertImage(bufferedImage, histogram.calculateHistogram(bufferedImage));

        StringBuilder sb = new StringBuilder();

        for(BufferedImage image : segmentation.extractCharacters(bufferedImage)) {
            int number = recognition.recognize(image);
            if (number != -1) {
                sb.append(number);
            } else {
                sb.append("?");
            }
        }

        return sb.toString();
    }
}
