package fer.hr.segmentation;

import fer.hr.preprocessing.histogram.IHistogram;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by jcular on 4/4/16.
 */
public class SegmentationWithPresetThreshold implements ISegmentation {

    private double HTHRESHOLD;
    private double VTHRESHOLD;
    private double INNERVTHRESHOLD;
    private double WIDTH;
    private double HEIGHT;

    private IHistogram rowSumHistogram;
    private IHistogram columnSumHistogram;

    public SegmentationWithPresetThreshold(IHistogram rowSumHistogram, IHistogram columnSumHistogram) {
        this(rowSumHistogram, columnSumHistogram, 80, 15, 5, 40, 40);
    }

    public SegmentationWithPresetThreshold(IHistogram rowSumHistogram, IHistogram columnSumHistogram,
                                           double horizontalThreshold, double verticalThreshold,
                                           double innerVerticalThreshold, double width, double height) {
        this.rowSumHistogram = rowSumHistogram;
        this.columnSumHistogram = columnSumHistogram;
        this.HTHRESHOLD = horizontalThreshold;
        this.VTHRESHOLD = verticalThreshold;
        this.INNERVTHRESHOLD = innerVerticalThreshold;
        this.WIDTH = width;
        this.HEIGHT = height;
    }

    @Override
    public List<BufferedImage> extractCharacters(BufferedImage image) {
        double[] horizontal = rowSumHistogram.calculateHistogram(image);
        int y1 = 0, y2 = 0;

        while (y1 < image.getHeight() && y2 < image.getHeight()) {
            if (horizontal[y1] >= HTHRESHOLD && y1 >= y2) {
                y2 = y1 + 1;
            } else if (y2 > y1 && horizontal[y2] >= HTHRESHOLD) {
                y2++;
            } else if (y2 > y1 && horizontal[y2] < HTHRESHOLD && y2 - y1 > HEIGHT) {
                break;
            } else {
                y1++;
            }
        }

        BufferedImage horizontalCut = image.getSubimage(0, y1, image.getWidth(), Math.abs(y2 - y1));

        double[] vertical = columnSumHistogram.calculateHistogram(horizontalCut);

        List<BufferedImage> characterImages = new ArrayList<>();
        int x1 = 0, x2 = 0;

        while (x1 < horizontalCut.getWidth() && x2 < horizontalCut.getWidth()) {
            if (vertical[x1] >= VTHRESHOLD && x1 >= x2) {
                x2 = x1 + 1;
            } else if (x2 > x1 && vertical[x2] >= INNERVTHRESHOLD) {
                x2++;
            } else if (x2 > x1 && vertical[x2] < VTHRESHOLD && x2 - x1 > WIDTH) {
                characterImages.add(horizontalCut.getSubimage(x1, 0, x2 - x1, horizontalCut.getHeight()));
                x1 = x2 + 1;
            } else {
                x1++;
            }
        }
        if (characterImages.size() == 0) {
            characterImages.add(horizontalCut);
        }
        return characterImages;
    }
}
